@@ -4,7 +4,7 @@ from victron_mqtt._victron_topics import topics
 
 def test_parsed_topic_with_pattern():
     # Find the TopicDescriptor with the desired topic
-    descriptor = next((t for t in topics if t.topic == "N/+/system/+/Relay/{relay}/State"), None)
+    descriptor = next((t for t in topics if t.topic == "N/{installation_id}/system/{device_id}/Relay/{relay}/State"), None)
     assert descriptor is not None, "TopicDescriptor with the specified topic not found"
 
     # Create a ParsedTopic instance
@@ -20,13 +20,12 @@ def test_parsed_topic_with_pattern():
     parsed_topic.finalize_topic_fields(descriptor)
     # Validate the ParsedTopic instance additional fields after matching description
     assert parsed_topic.key_values["relay"] == "1", "Relay ID should match"
-    assert parsed_topic.name == "Relay 1 state", "Name should match"
     assert parsed_topic.short_id == "system_relay_1", "Short ID should match"
 
 
 def test_parsed_topic_with_phase():
     # Find the TopicDescriptor with the desired topic
-    descriptor = next((t for t in topics if t.topic == "N/+/system/+/Ac/Genset/{phase}/Power"), None)
+    descriptor = next((t for t in topics if t.topic == "N/{installation_id}/system/{device_id}/Ac/Genset/{phase}/Power"), None)
     assert descriptor is not None, "TopicDescriptor with the specified topic not found"
 
     # Create a ParsedTopic instance
@@ -42,12 +41,11 @@ def test_parsed_topic_with_phase():
     parsed_topic.finalize_topic_fields(descriptor)
     # Validate the ParsedTopic instance additional fields after matching description
     assert parsed_topic.key_values["phase"] == "L1", "Phase should match"
-    assert parsed_topic.name == "Genset Load L1", "Name should match"
     assert parsed_topic.short_id == "system_generator_load_L1", "Short ID should match"
 
 def test_parsed_topic_with_next_phase():
     # Find the TopicDescriptor with the desired topic
-    descriptor = next((t for t in topics if t.topic == "N/+/grid/+/Ac/{phase}/VoltageLineToLine"), None)
+    descriptor = next((t for t in topics if t.topic == "N/{installation_id}/grid/{device_id}/Ac/{phase}/VoltageLineToLine"), None)
     assert descriptor is not None, "TopicDescriptor with the specified topic not found"
 
     # Create a ParsedTopic instance
@@ -63,12 +61,11 @@ def test_parsed_topic_with_next_phase():
     parsed_topic.finalize_topic_fields(descriptor)
     # Validate the ParsedTopic instance additional fields after matching description
     assert parsed_topic.key_values["phase"] == "L3", "Phase should match"
-    assert parsed_topic.name == "Grid voltage L3 to L1", "Name should match"
     assert parsed_topic.short_id == "grid_voltage_L3_L1", "Short ID should match"
 
 def test_parsed_topic_with_phase_and_placeholder():
     # Find the TopicDescriptor with the desired topic
-    descriptor = next((t for t in topics if t.topic == "N/+/multi/+/Ac/Out/{output}/{phase}/I"), None)
+    descriptor = next((t for t in topics if t.topic == "N/{installation_id}/multi/{device_id}/Ac/Out/{output}/{phase}/I"), None)
     assert descriptor is not None, "TopicDescriptor with the specified topic not found"
 
     # Create a ParsedTopic instance
@@ -84,12 +81,11 @@ def test_parsed_topic_with_phase_and_placeholder():
     parsed_topic.finalize_topic_fields(descriptor)
     # Validate the ParsedTopic instance additional fields after matching description
     assert parsed_topic.key_values["phase"] == "L1", "Phase should match"
-    assert parsed_topic.name == "AC Out 1 Current on L1", "Name should match"
     assert parsed_topic.short_id == "multirssolar_acout_1_current_L1", "Short ID should match"
 
 def test_settings_parsed_topic():
     # Find the TopicDescriptor with the desired topic
-    descriptor = next((t for t in topics if t.topic == "N/+/settings/+/Settings/CGwacs/AcPowerSetPoint"), None)
+    descriptor = next((t for t in topics if t.topic == "N/{installation_id}/settings/{device_id}/Settings/CGwacs/AcPowerSetPoint"), None)
     assert descriptor is not None, "TopicDescriptor with the specified topic not found"
 
     # Create a ParsedTopic instance
@@ -104,13 +100,12 @@ def test_settings_parsed_topic():
 
     parsed_topic.finalize_topic_fields(descriptor)
     # Validate the ParsedTopic instance additional fields after matching description
-    assert parsed_topic.name == "AC Power Setpoint", "Name should match"
     assert parsed_topic.short_id == "system_ac_power_set_point", "Short ID should match"
 
 
 def test_settings_parsed_topic_2():
     # Find the TopicDescriptor with the desired topic
-    descriptor = next((t for t in topics if t.topic == "N/+/settings/+/Settings/SystemSetup/MaxChargeCurrent"), None)
+    descriptor = next((t for t in topics if t.topic == "N/{installation_id}/settings/{device_id}/Settings/SystemSetup/MaxChargeCurrent"), None)
     assert descriptor is not None, "TopicDescriptor with the specified topic not found"
 
     # Create a ParsedTopic instance
@@ -125,5 +120,4 @@ def test_settings_parsed_topic_2():
 
     parsed_topic.finalize_topic_fields(descriptor)
     # Validate the ParsedTopic instance additional fields after matching description
-    assert parsed_topic.name == "ESS max charge current", "Name should match"
     assert parsed_topic.short_id == "system_ess_max_charge_current", "Short ID should match"
