@@ -132,7 +132,7 @@ async def test_hub_message_handling(create_mocked_hub):
     inject_message(hub, "N/device/123/metric/456", "{\"value\": 42}")
 
     # Validate the Hub's state
-    assert len(hub._devices) == 1, "No devices should be created"
+    assert len(hub.devices) == 1, "No devices should be created"
 
     await finalize_injection(hub)
 
@@ -146,10 +146,10 @@ async def test_phase_message(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 2, f"Expected 2 device, got {len(hub._devices)}"
+    assert len(hub.devices) == 2, f"Expected 2 device, got {len(hub.devices)}"
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_grid_30"]
+    device = hub.devices["123_grid_30"]
     assert device.device_type == DeviceType.GRID, f"Expected metric type to be 'grid', got {device.device_type}"
     metric = device.get_metric_from_unique_id("123_grid_30_grid_energy_forward_L1")
     assert metric is not None, "Metric should exist in the device"
@@ -173,10 +173,10 @@ async def test_placeholder_message(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
+    assert len(hub.devices) == 1, f"Expected 1 device, got {len(hub.devices)}"
 
     # Validate that the device has the metric we published
-    device =hub._devices["123_system_0"]
+    device =hub.devices["123_system_0"]
     metric = device.get_metric_from_unique_id("123_system_0_system_relay_0")
     assert metric is not None, "Metric should exist in the device"
     assert metric.value == GenericOnOff.On, f"Expected metric value to be GenericOnOff.On, got {metric.value}"
@@ -193,10 +193,10 @@ async def test_dynamic_min_max_message(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
+    assert len(hub.devices) == 1, f"Expected 1 device, got {len(hub.devices)}"
 
     # Validate that the device has the metric we published
-    device = list(hub._devices.values())[0]
+    device = list(hub.devices.values())[0]
     writable_metric = device.get_metric_from_unique_id("123_system_0_system_ac_power_set_point")
     assert writable_metric is not None, "WritableMetric should exist in the device"
     assert isinstance(writable_metric, WritableMetric), f"Expected writable_metric to be of type WritableMetric, got {type(writable_metric)}"
@@ -215,10 +215,10 @@ async def test_number_message(create_mocked_hub):
     await finalize_injection(hub, False)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 2, f"Expected 2 device, got {len(hub._devices)}"
+    assert len(hub.devices) == 2, f"Expected 2 device, got {len(hub.devices)}"
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_evcharger_170"]
+    device = hub.devices["123_evcharger_170"]
     writable_metric = device.get_metric_from_unique_id("123_evcharger_170_evcharger_set_current")
     assert isinstance(writable_metric, WritableMetric), f"Expected writable_metric to be of type WritableMetric, got {type(writable_metric)}"
     assert writable_metric.value == 100, f"Expected writable_metric value to be 100, got {writable_metric.value}"
@@ -259,10 +259,10 @@ async def test_placeholder_adjustable_on(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 2, f"Expected 2 device, got {len(hub._devices)}"
+    assert len(hub.devices) == 2, f"Expected 2 device, got {len(hub.devices)}"
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_vebus_170"]
+    device = hub.devices["123_vebus_170"]
     assert len(device._metrics) == 1, f"Expected 1 metrics, got {len(device._metrics)}"
     writable_metric = device.get_metric_from_unique_id("123_vebus_170_vebus_inverter_current_limit")
     assert isinstance(writable_metric, WritableMetric), f"Expected writable_metric to be of type WritableMetric, got {type(writable_metric)}"
@@ -281,10 +281,10 @@ async def test_placeholder_adjustable_off(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 2, f"Expected 2 device, got {len(hub._devices)}"
+    assert len(hub.devices) == 2, f"Expected 2 device, got {len(hub.devices)}"
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_vebus_170"]
+    device = hub.devices["123_vebus_170"]
     assert len(device._metrics) == 1, f"Expected 1 metrics, got {len(device._metrics)}"
     metric = device.get_metric_from_unique_id("123_vebus_170_vebus_inverter_current_limit")
     assert not isinstance(metric, WritableMetric), f"Expected metric to be of type Metric, got {type(metric)}"
@@ -303,7 +303,7 @@ async def test_placeholder_adjustable_on_reverse(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_vebus_170"]
+    device = hub.devices["123_vebus_170"]
     assert len(device._metrics) == 1, f"Expected 1 metrics, got {len(device._metrics)}"
     writable_metric = device.get_metric_from_unique_id("123_vebus_170_vebus_inverter_current_limit")
     assert isinstance(writable_metric, WritableMetric), f"Expected writable_metric to be of type WritableMetric, got {type(writable_metric)}"
@@ -322,7 +322,7 @@ async def test_placeholder_adjustable_off_reverse(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_vebus_170"]
+    device = hub.devices["123_vebus_170"]
     assert len(device._metrics) == 1, f"Expected 1 metrics, got {len(device._metrics)}"
     writable_metric = device.get_metric_from_unique_id("123_vebus_170_vebus_inverter_current_limit")
     assert not isinstance(writable_metric, WritableMetric), f"Expected writable_metric to be of type WritableMetric, got {type(writable_metric)}"
@@ -342,7 +342,7 @@ async def test_today_message(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_solarcharger_290"]
+    device = hub.devices["123_solarcharger_290"]
     assert len(device._metrics) == 2, f"Expected 2 metrics, got {len(device._metrics)}"
 
     metric = device.get_metric_from_unique_id("123_solarcharger_290_solarcharger_max_power_today")
@@ -375,7 +375,7 @@ async def test_expend_message(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_switch_170"]
+    device = hub.devices["123_switch_170"]
     metric = device.get_metric_from_unique_id("123_switch_170_switch_2_state")
     assert metric is not None, "Metric should exist in the device"
     assert metric.generic_short_id == "switch_{output}_state"
@@ -392,7 +392,7 @@ async def test_expend_message_2(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_battery_170"]
+    device = hub.devices["123_battery_170"]
     metric = device.get_metric_from_unique_id("123_battery_170_battery_cell_3_voltage")
     assert metric is not None, "Metric should exist in the device"
     assert metric.generic_short_id == "battery_cell_{cell_id}_voltage"
@@ -410,7 +410,7 @@ async def test_same_message_events(create_mocked_hub):
     await finalize_injection(hub, False)
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_grid_30"]
+    device = hub.devices["123_grid_30"]
     metric = device.get_metric_from_unique_id("123_grid_30_grid_energy_forward_L1")
     assert metric is not None, "Metric should exist in the device"
     assert metric.value == 42, f"Expected metric value to be 42, got {metric.value}"
@@ -436,7 +436,7 @@ async def test_existing_installation_id(create_mocked_hub_with_installation_id):
     await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_switch_170"]
+    device = hub.devices["123_switch_170"]
     metric = device.get_metric_from_unique_id("123_switch_170_switch_2_state")
     assert metric is not None, "Metric should exist in the device"
     assert metric.generic_short_id == "switch_{output}_state"
@@ -454,7 +454,7 @@ async def test_multiple_hubs(create_mocked_hub_with_installation_id, create_mock
     await finalize_injection(hub1, disconnect=False)
 
     # Validate that the device has the metric we published
-    device1 = hub1._devices["123_switch_170"]
+    device1 = hub1.devices["123_switch_170"]
     metric1 = device1.get_metric_from_unique_id("123_switch_170_switch_2_state")
     assert metric1 is not None, "Metric should exist in the device"
     assert metric1.generic_short_id == "switch_{output}_state"
@@ -467,10 +467,10 @@ async def test_multiple_hubs(create_mocked_hub_with_installation_id, create_mock
     await finalize_injection(hub2, disconnect=False)
 
     # Validate the Hub's state
-    assert len(hub2._devices) == 1, f"Expected 1 device, got {len(hub1._devices)}"
+    assert len(hub2.devices) == 1, f"Expected 1 device, got {len(hub1.devices)}"
 
     # Validate that the device has the metric we published
-    device2 = hub2._devices["123_switch_170"]
+    device2 = hub2.devices["123_switch_170"]
     metric2 = device2.get_metric_from_unique_id("123_switch_170_switch_2_state")
     assert metric2 is not None, "Metric should exist in the device"
     assert metric2.generic_short_id == "switch_{output}_state"
@@ -490,7 +490,7 @@ async def test_float_precision(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_system_170"]
+    device = hub.devices["123_system_170"]
     metric = device.get_metric_from_unique_id("123_system_170_system_dc_consumption")
     assert metric is not None, "Metric should exist in the device"
     assert metric.value == 1.1, f"Expected metric value to be 1.1, got {metric.value}"
@@ -505,7 +505,7 @@ async def test_float_precision_none(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_gps_170"]
+    device = hub.devices["123_gps_170"]
     metric = device.get_metric_from_unique_id("123_gps_170_gps_latitude")
     assert metric is not None, "Metric should exist in the device"
     assert metric.value == 1.0123456789, f"Expected metric value to be 1.0123456789, got {metric.value}"
@@ -529,8 +529,8 @@ async def test_new_metric(create_mocked_hub):
     await asyncio.sleep(0.1)  # Allow event loop to process the callback
 
     # Validate that the on_new_metric callback was called
-    hub.on_new_metric.assert_any_call(hub, hub._devices["123_system_170"], hub._devices["123_system_170"].get_metric_from_unique_id("123_system_170_system_dc_consumption"))
-    hub.on_new_metric.assert_called_with(hub, hub._devices["123_gps_170"], hub._devices["123_gps_170"].get_metric_from_unique_id("123_gps_170_gps_latitude"))
+    hub.on_new_metric.assert_any_call(hub, hub.devices["123_system_170"], hub.devices["123_system_170"].get_metric_from_unique_id("123_system_170_system_dc_consumption"))
+    hub.on_new_metric.assert_called_with(hub, hub.devices["123_gps_170"], hub.devices["123_gps_170"].get_metric_from_unique_id("123_gps_170_gps_latitude"))
     assert hub.on_new_metric.call_count == 2, "on_new_metric should be called exactly twice"
 
     # Check that we got the callback only once
@@ -540,7 +540,7 @@ async def test_new_metric(create_mocked_hub):
     assert hub.on_new_metric.call_count == 2, "on_new_metric should be called exactly twice"
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_system_170"]
+    device = hub.devices["123_system_170"]
     metric = device.get_metric_from_unique_id("123_system_170_system_dc_consumption")
     assert metric is not None, "Metric should exist in the device"
     assert metric.value == 1.1, f"Expected metric value to be 1.1, got {metric.value}"
@@ -558,7 +558,7 @@ async def test_experimental_metrics_not_created_by_default(create_mocked_hub):
     await finalize_injection(hub)
 
     # The experimental topic should not have created a device or metric
-    assert "123_generator_170" not in hub._devices, "Experimental topic should not create devices/metrics when operation_mode is not EXPERIMENTAL"
+    assert "123_generator_170" not in hub.devices, "Experimental topic should not create devices/metrics when operation_mode is not EXPERIMENTAL"
 
 @pytest.mark.asyncio
 async def test_experimental_metrics_created_when_needed(create_mocked_hub_experimental):
@@ -570,7 +570,7 @@ async def test_experimental_metrics_created_when_needed(create_mocked_hub_experi
     await finalize_injection(hub)
 
     # The experimental topic should not have created a device or metric
-    assert "123_generator_170" in hub._devices, "Experimental topic should not create devices/metrics when operation_mode is not EXPERIMENTAL"
+    assert "123_generator_170" in hub.devices, "Experimental topic should not create devices/metrics when operation_mode is not EXPERIMENTAL"
 
 @pytest.mark.asyncio
 async def test_read_only_creates_plain_metrics(create_mocked_hub_read_only):
@@ -581,8 +581,8 @@ async def test_read_only_creates_plain_metrics(create_mocked_hub_read_only):
     await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert "123_evcharger_170" in hub._devices, "Device should be created"
-    device = hub._devices["123_evcharger_170"]
+    assert "123_evcharger_170" in hub.devices, "Device should be created"
+    device = hub.devices["123_evcharger_170"]
     metric = device.get_metric_from_unique_id("123_evcharger_170_evcharger_set_current")
     assert metric is not None, "Metric should exist in the device"
     assert not isinstance(metric, WritableMetric), "In READ_ONLY mode the metric should NOT be a WritableMetric"
@@ -644,8 +644,8 @@ async def test_filtered_message(create_mocked_hub_with_device_filter):
     inject_message(hub, "N/123/evcharger/170/SetCurrent", "{\"value\": 100}")
 
     # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
-    assert "123_system_0" in hub._devices, "Expected only the system device to exist"
+    assert len(hub.devices) == 1, f"Expected 1 device, got {len(hub.devices)}"
+    assert "123_system_0" in hub.devices, "Expected only the system device to exist"
 
 @pytest.mark.asyncio
 async def test_filtered_message_system(create_mocked_hub_with_device_filter):
@@ -657,8 +657,8 @@ async def test_filtered_message_system(create_mocked_hub_with_device_filter):
     await finalize_injection(hub)
 
     # Validate the Hub's state - system device exists but has no metrics due to filtering
-    assert len(hub._devices) == 1, f"Expected 1 device (system device), got {len(hub._devices)}"
-    system_device = hub._devices["123_system_0"]
+    assert len(hub.devices) == 1, f"Expected 1 device (system device), got {len(hub.devices)}"
+    system_device = hub.devices["123_system_0"]
     assert len(system_device._metrics) == 0, f"Expected 0 metrics on system device due to filtering, got {len(system_device._metrics)}"
 
 @pytest.mark.asyncio
@@ -671,8 +671,8 @@ async def test_no_filtered_message_placeholder(create_mocked_hub_with_device_fil
     await finalize_injection(hub)
 
     # Validate the Hub's state - only system device exists, generator message was filtered
-    assert len(hub._devices) == 2, f"Expected 2 devices (system and generator1), got {len(hub._devices)}"
-    system_device = hub._devices["123_system_0"]
+    assert len(hub.devices) == 2, f"Expected 2 devices (system and generator1), got {len(hub.devices)}"
+    system_device = hub.devices["123_system_0"]
     assert system_device.device_type.value[0] == "system", f"Expected system device, got {system_device.device_type.value}"
 
 
@@ -686,8 +686,8 @@ async def test_filtered_message_placeholder(create_mocked_hub_with_device_filter
     await finalize_injection(hub)
 
     # Validate the Hub's state - only system device exists, generator message was filtered
-    assert len(hub._devices) == 1, f"Expected 1 device (system device), got {len(hub._devices)}"
-    system_device = hub._devices["123_system_0"]
+    assert len(hub.devices) == 1, f"Expected 1 device (system device), got {len(hub.devices)}"
+    system_device = hub.devices["123_system_0"]
     assert system_device.device_type.value[0] == "system", f"Expected system device, got {system_device.device_type.value}"
 
 
@@ -700,10 +700,10 @@ async def test_remote_name_dont_exists(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 2, f"Expected 2 device, got {len(hub._devices)}"
+    assert len(hub.devices) == 2, f"Expected 2 device, got {len(hub.devices)}"
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_switch_170"]
+    device = hub.devices["123_switch_170"]
     assert len(device._metrics) == 1, f"Expected 1 metrics, got {len(device._metrics)}"
     metric = device.get_metric_from_unique_id("123_switch_170_switch_1_state")
     assert metric is not None, "metric should exist in the device"
@@ -722,10 +722,10 @@ async def test_remote_name_exists(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 2, f"Expected 2 device, got {len(hub._devices)}"
+    assert len(hub.devices) == 2, f"Expected 2 device, got {len(hub.devices)}"
 
     # Validate that the device has the metric we published
-    device = hub._devices["123_switch_170"]
+    device = hub.devices["123_switch_170"]
     assert len(device._metrics) == 2, f"Expected 2 metrics, got {len(device._metrics)}"
     metric = device.get_metric_from_unique_id("123_switch_170_switch_1_state")
     assert metric is not None, "metric should exist in the device"
@@ -734,6 +734,61 @@ async def test_remote_name_exists(create_mocked_hub):
     assert metric.value == GenericOnOff.On, f"Expected metric value to be 1, got {metric.value}"
     assert metric.key_values["output"] == "bla"
 
+@pytest.mark.asyncio
+async def test_remote_name_exists_twodevices(create_mocked_hub):
+    """Test that the Hub correctly updates its internal state based on MQTT messages."""
+    hub: Hub = create_mocked_hub
+
+    inject_message(hub, "N/123/switch/170/SwitchableOutput/output_1/State", "{\"value\": 1}")
+    inject_message(hub, "N/123/switch/170/SwitchableOutput/output_1/Settings/CustomName", "{\"value\": \"bla\"}")
+    inject_message(hub, "N/123/switch/155/SwitchableOutput/output_1/State", "{\"value\": 1}")
+    inject_message(hub, "N/123/switch/155/SwitchableOutput/output_1/Settings/CustomName", "{\"value\": \"foo\"}")
+    await finalize_injection(hub)
+
+    # Validate the Hub's state
+    assert len(hub.devices) == 3, f"Expected 3 device, got {len(hub.devices)}"
+
+    # Validate that the device has the metric we published
+    device = hub.devices["123_switch_170"]
+    assert len(device._metrics) == 2, f"Expected 2 metrics, got {len(device._metrics)}"
+    metric = device.get_metric_from_unique_id("123_switch_170_switch_1_state")
+    assert metric is not None, "metric should exist in the device"
+    assert metric.name == "Switch bla State", "Expected metric name to be 'Switch bla State', got {metric.name}"
+    assert metric.generic_name == "Switch {output} State", "Expected metric name to be 'Switch {output} State', got {metric.name}"
+    assert metric.value == GenericOnOff.On, f"Expected metric value to be 1, got {metric.value}"
+    assert metric.key_values["output"] == "bla"
+
+    device = hub.devices["123_switch_155"]
+    assert len(device._metrics) == 2, f"Expected 2 metrics, got {len(device._metrics)}"
+    metric = device.get_metric_from_unique_id("123_switch_155_switch_1_state")
+    assert metric is not None, "metric should exist in the device"
+    assert metric.name == "Switch foo State", "Expected metric name to be 'Switch foo State', got {metric.name}"
+    assert metric.generic_name == "Switch {output} State", "Expected metric name to be 'Switch {output} State', got {metric.name}"
+    assert metric.value == GenericOnOff.On, f"Expected metric value to be 1, got {metric.value}"
+    assert metric.key_values["output"] == "foo"
+
+@pytest.mark.asyncio
+async def test_remote_name_exists_2(create_mocked_hub):
+    """Test that the Hub correctly updates its internal state based on MQTT messages."""
+    hub: Hub = create_mocked_hub
+
+    inject_message(hub, "N/123/solarcharger/170/Pv/2/Name", "{\"value\": \"bar\"}")
+    inject_message(hub, "N/123/solarcharger/170/Pv/2/P", "{\"value\": 1}")
+    await finalize_injection(hub)
+
+    # Validate the Hub's state
+    assert len(hub.devices) == 2, f"Expected 2 device, got {len(hub.devices)}"
+
+    # Validate that the device has the metric we published
+    device = hub.devices["123_solarcharger_170"]
+    assert len(device._metrics) == 2, f"Expected 2 metrics, got {len(device._metrics)}"
+    metric = device.get_metric_from_unique_id("123_solarcharger_170_solarcharger_tracker_2_power")
+    assert metric is not None, "metric should exist in the device"
+    assert metric.name == "PV Tracker bar Power", "Expected metric name to be 'PV Tracker bar Power', got {metric.name}"
+    assert metric.generic_name == "PV Tracker {tracker} Power", "Expected metric name to be 'PV Tracker {tracker} Power', got {metric.name}"
+    assert metric.value == 1, f"Expected metric value to be 1, got {metric.value}"
+    assert metric.key_values["tracker"] == "bar"
+
 @pytest.mark.asyncio
 async def test_on_connect_sets_up_subscriptions():
     """Test that subscriptions are set up after _on_connect callback."""
@@ -774,7 +829,7 @@ async def test_null_message(create_mocked_hub):
     await finalize_injection(hub)
 
     # Validate the Hub's state - only system device exists, evcharger message was filtered
-    assert len(hub._devices) == 2, f"Expected 2 device (system device), got {len(hub._devices)}"
-    device = hub._devices["123_evcharger_170"]
+    assert len(hub.devices) == 2, f"Expected 2 device (system device), got {len(hub.devices)}"
+    device = hub.devices["123_evcharger_170"]
     assert device.metrics == [], f"Expected 0 metrics on evcharger device due to null message, got {len(device._metrics)}"
 
