@@ -977,6 +977,21 @@ topics: List[TopicDescriptor] = [
         name="Total PV Yield",
         metric_type=MetricType.ENERGY,
     ),
+    # Platform topics
+    TopicDescriptor(
+        topic="N/{installation_id}/platform/{device_id}/Firmware/Installed/Version",
+        message_type=MetricKind.SENSOR,
+        value_type=ValueType.STRING,
+        short_id="platform_venus_firmware_installed_version",
+        name="Firmware Venus Installed Version",
+    ),
+    TopicDescriptor(
+        topic="N/{installation_id}/platform/{device_id}/Firmware/Online/AvailableVersion",
+        message_type=MetricKind.SENSOR,
+        value_type=ValueType.STRING,
+        short_id="platform_venus_firmware_available_version",
+        name="Firmware Venus Available Version",
+    ),
     # PV Inverter topics
     TopicDescriptor(
         topic="N/{installation_id}/pvinverter/{device_id}/Ac/Energy/Forward",
@@ -1836,6 +1851,14 @@ topics: List[TopicDescriptor] = [
         value_type=ValueType.ENUM,
         enum=GenericOnOff,
     ),
+    TopicDescriptor(
+        topic="N/{installation_id}/system/{device_id}/SystemState/State",
+        message_type=MetricKind.SENSOR,
+        short_id="system_state",
+        name="System state",
+        value_type=ValueType.ENUM,
+        enum=State,
+    ),
     # Tank topics
     TopicDescriptor(
         topic="N/{installation_id}/tank/{device_id}/BatteryVoltage",
