@@ -15,6 +15,7 @@ class MetricKind(Enum):
     SWITCH = "switch"
     SELECT = "select"
     NUMBER = "number"
+    SERVICE = "service"
 
 
 class MetricNature(Enum):
@@ -58,12 +59,20 @@ class ValueType(Enum):
     STRING = "str"
     ENUM = "enum"
     EPOCH = "epoch"
+    INT_SECONDS_TO_HOURS = "int_seconds_to_hours"
 
 class RangeType(Enum):
     """Range types for numeric values."""
     STATIC = "static"  # Static range, e.g., fixed values
     DYNAMIC = "dynamic"  # Dynamic range, e.g., depends on device model
 
+class OperationMode(Enum):
+    """Enum for operation modes."""
+    READ_ONLY = "read_only"
+    FULL = "full"
+    EXPERIMENTAL = "experimental"
+
+
 
 PLACEHOLDER_PHASE = "{phase}"
 PLACEHOLDER_NEXT_PHASE = "{next_phase}"
