@@ -17,9 +17,6 @@ def topic_to_device_type(topic_parts: list[str]) -> DeviceType | None:
     if len(topic_parts) < 2:
         return None
     native_device_type = topic_parts[2]
-    # For whatever reason some system  send topics to  N/+/platform/0/ProductName
-    if native_device_type == "platform":
-        native_device_type = "system"
     # for settings like N/+/settings/0/Settings/CGwacs/AcPowerSetPoint
     if native_device_type == "settings":
         native_device_type = topic_parts[5]
